/*=============================================================================*
 *  Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws.interop.client;

import org.apache.ws.addressing.XmlBeansEndpointReference;
import org.apache.ws.muws.v1_0.MuwsConstants;
import org.apache.ws.muws.v1_0.capability.ConfigurationCapability;
import org.apache.ws.muws.v1_0.capability.ManageabilityCharacteristicsCapability;
import org.apache.ws.muws.v1_0.capability.MetricsCapability;
import org.apache.ws.muws.v1_0.capability.OperationalStatusCapability;
import org.apache.ws.resource.example.blackberry.BlackberryPropertyQNames;
import org.apache.ws.resource.properties.v2004_06.ResourcePropertiesConstants;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.ws.util.test.PortListen;
import org.apache.xmlbeans.XmlDateTime;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ManagementEventType;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.OperationalStatusDocument;
import org.oasisOpen.docs.wsrf.x2004.x06.wsrfWSResourceProperties12Draft01.ResourcePropertyValueChangeNotificationType;
import org.xmlsoap.schemas.ws.x2003.x03.addressing.EndpointReferenceDocument;
import rim.pda.DateDocument;
import rim.pda.TimeDocument;
import rim.pda.TotalCodeModuleSizeDocument;

import javax.xml.namespace.QName;
import java.net.URL;

/**
 * A test case containing a test for each act in the "Small Device Management"
 * playbook, as defined in the WSDM 1.0 interop scenarios doc.
 *
 * @author Ian Springer
 */
public class SmallDeviceMgmtTestCase extends AbstractWsdmInteropTestCase
{

    public static final String SYSPROP_BLACKBERRY_EPR = "blackberry.epr";

    private static final String HP_BLACKBERRY_EPR = "http://cvs.apache.org/~ips/interop/blackberry-epr.xml";
    private static final String IBM_BLACKBERRY_EPR = "http://66.57.65.244:4400/ettk/tinywsdmDemo/BBEPR/RIM2100000A";

    private static final String BLACKBERRY_EPR = System.getProperty( SYSPROP_BLACKBERRY_EPR, HP_BLACKBERRY_EPR );

    private static final String NSURI_BLACKBERRY = "http://pda.rim";
    private static final String NSPREFIX_BLACKBERRY = "bbry";
    private static final String CAPABILITY_URI_BLACKBERRY_CONFIGURATION = NSURI_BLACKBERRY + "/BlackberryConfiguration";
    private static final String CAPABILITY_URI_BLACKBERRY_METRICS = NSURI_BLACKBERRY + "/BlackberryMetrics";
    private static final QName PROP_NAME_DATE = new QName( NSURI_BLACKBERRY, "date", NSPREFIX_BLACKBERRY );
    private static final QName PROP_NAME_TIME = new QName( NSURI_BLACKBERRY, "time", NSPREFIX_BLACKBERRY );

    private ResourceStub m_resource;

    protected void setUp() throws Exception
    {
        super.setUp();
        EndpointReferenceDocument blackberryEprDoc = getEndpointReference( new URL( BLACKBERRY_EPR ) );
        if ( DEBUG ) { System.out.println( "*** Using Blackberry EPR:\n" + blackberryEprDoc ); }
        m_resource = new ResourceStub( new XmlBeansEndpointReference(blackberryEprDoc.getEndpointReference()) );        
    }

    public void testMuwsIdentityCharacteristics() throws FaultException
    {
        performMuwsIdentityCharacteristicsAct( m_resource );
    }

    public void testBlackberryMetrics() throws FaultException
    {
        XmlObject[] capabilityElems = m_resource.getResourceProperty( ManageabilityCharacteristicsCapability.PROP_NAME_MANAGEABILITY_CAPABILITY );
        assertContainsURI( capabilityElems, MetricsCapability.URI );
        assertContainsURI( capabilityElems, CAPABILITY_URI_BLACKBERRY_METRICS );

        XmlObject currentTimePropElem = getSingleProperty( m_resource, MetricsCapability.PROP_NAME_CURRENT_TIME );
        assertTrue( currentTimePropElem instanceof XmlDateTime );
        XmlObject allocatedStoragePropElem = getSingleProperty( m_resource, BlackberryPropertyQNames.ALLOCATEDSTORAGE );
        // TODO: check type & metric attribs
        XmlObject freeStoragePropElem = getSingleProperty( m_resource, BlackberryPropertyQNames.FREESTORAGE );
        // TODO: check type & metric attribs
        XmlObject totalCodeModuleSizePropElem = getSingleProperty( m_resource, BlackberryPropertyQNames.TOTALCODEMODULESIZE );
        // TODO: check type & metric attribs
    }

    public void testBlackberryConfiguration() throws FaultException
    {
        XmlObject[] capabilityElems = m_resource.getResourceProperty( ManageabilityCharacteristicsCapability.PROP_NAME_MANAGEABILITY_CAPABILITY );
        assertContainsURI( capabilityElems, ConfigurationCapability.URI );
        assertContainsURI( capabilityElems, CAPABILITY_URI_BLACKBERRY_CONFIGURATION );

        // set date prop
        DateDocument dateDoc = DateDocument.Factory.newInstance();
        DateDocument.Date newDate = dateDoc.addNewDate();
        newDate.setDay( 24 );
        newDate.setMonth( 8 );
        newDate.setYear( 1975 );
        setSingleProperty( m_resource, dateDoc );

        // set time prop
        TimeDocument timeDoc = TimeDocument.Factory.newInstance();
        TimeDocument.Time newTime = timeDoc.addNewTime();
        newTime.setHour( 6 );
        newTime.setMinute( 30 );
        setSingleProperty( m_resource, timeDoc );

        // get date prop and validate value is as we set it
        XmlObject datePropElem = getSingleProperty( m_resource, PROP_NAME_DATE );
        assertTrue( datePropElem instanceof DateDocument.Date );
        DateDocument.Date date = (DateDocument.Date)datePropElem;
        assertEquals( newDate.getDay(), date.getDay() );
        assertEquals( newDate.getMonth(), date.getMonth() );
        assertEquals( newDate.getYear(), date.getYear() );

        // get time prop and validate value is as we set it
        XmlObject timePropElem = getSingleProperty( m_resource, PROP_NAME_TIME );
        assertTrue( timePropElem instanceof TimeDocument.Time );
        TimeDocument.Time time = (TimeDocument.Time)timePropElem;
        assertEquals( newTime.getHour(), time.getHour() );
        assertEquals( newTime.getMinute(), time.getMinute() );
    }

    public void testMuwsStatus() throws FaultException
    {
        performMuwsStatusAct( m_resource );
    }

    public void testMuwsMetricChange() throws Exception
    {
        XmlObject[] capabilityElems = m_resource.getResourceProperty( ManageabilityCharacteristicsCapability.PROP_NAME_MANAGEABILITY_CAPABILITY );
        assertContainsURI( capabilityElems, MetricsCapability.URI );

        m_resource.subscribe( s_consumerURL, new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, MetricsCapability.TOPIC_NAME, MuwsConstants.NSPREFIX_MUWS_PART2_TOPICS ) );

        PortListen notifListener = new PortListen( NOTIF_LISTENER_PORT, NOTIF_LISTENER_TIMEOUT );

        TotalCodeModuleSizeDocument totalCodeModuleSizeDocument = TotalCodeModuleSizeDocument.Factory.newInstance();
        totalCodeModuleSizeDocument.setTotalCodeModuleSize( 1111 );
        setSingleProperty( m_resource, totalCodeModuleSizeDocument );

        ManagementEventType mgmtEvent = waitForManagementEvent( notifListener );

        XmlObject propChangedEvent = XmlBeanUtils.getChildElements(mgmtEvent, new QName(ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "ResourcePropertyValueChangeNotification"))[0];
        assertNotNull("ManagementEvent does not contain a ResourcePropertyValueChangeNotification", propChangedEvent);
    }

    public void testBlackberryCodeModuleSizeChangeEvent() throws Exception
    {
        XmlObject[] capabilityElems = m_resource.getResourceProperty( ManageabilityCharacteristicsCapability.PROP_NAME_MANAGEABILITY_CAPABILITY );
        assertContainsURI( capabilityElems, MetricsCapability.URI );

        m_resource.subscribe( s_consumerURL, new QName( NSURI_BLACKBERRY, "CodeModuleSize", NSPREFIX_BLACKBERRY ) );

        PortListen notifListener = new PortListen( NOTIF_LISTENER_PORT, NOTIF_LISTENER_TIMEOUT );

        TotalCodeModuleSizeDocument totalCodeModuleSizeDocument = TotalCodeModuleSizeDocument.Factory.newInstance();
        totalCodeModuleSizeDocument.setTotalCodeModuleSize( 9999 );
        setSingleProperty( m_resource, totalCodeModuleSizeDocument );

        ManagementEventType mgmtEvent = waitForManagementEvent( notifListener );

        XmlObject propChangedEvent = XmlBeanUtils.getChildElements(mgmtEvent, new QName(ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "ResourcePropertyValueChangeNotification"))[0];
        assertNotNull("ManagementEvent does not contain a ResourcePropertyValueChangeNotification", propChangedEvent);
    }

    public void testMuwsStatusChangeEvent() throws Exception
    {
        XmlObject[] capabilityPropElems = m_resource.getResourceProperty(
                ManageabilityCharacteristicsCapability.PROP_NAME_MANAGEABILITY_CAPABILITY );
        assertContainsURI( capabilityPropElems, OperationalStatusCapability.URI );

        // init status to Available
        OperationalStatusDocument operationalStatusDoc = OperationalStatusDocument.Factory.newInstance();
        operationalStatusDoc.setOperationalStatus( OperationalStatusDocument.OperationalStatus.AVAILABLE );
        setSingleProperty( m_resource, operationalStatusDoc );

        m_resource.subscribe( s_consumerURL, new QName( MuwsConstants.NSURI_MUWS_PART2_TOPICS, OperationalStatusCapability.TOPIC_NAME, MuwsConstants.NSPREFIX_MUWS_PART2_TOPICS ) );
        PortListen notifListener = new PortListen( NOTIF_LISTENER_PORT, NOTIF_LISTENER_TIMEOUT );

        // change status to Unavailable
        operationalStatusDoc.setOperationalStatus( OperationalStatusDocument.OperationalStatus.UNAVAILABLE );
        setSingleProperty( m_resource, operationalStatusDoc );

        // make sure we receive a propChanged mgmtEvent
        ManagementEventType mgmtEvent = waitForManagementEvent( notifListener );
        XmlObject[] propChangedEvents = XmlBeanUtils.getChildElements(mgmtEvent, new QName(ResourcePropertiesConstants.NSURI_WSRP_SCHEMA, "ResourcePropertyValueChangeNotification"));
        assertEquals( "ManagementEvent does not contain a single ResourcePropertyValueChangeNotification", 1, propChangedEvents.length );
        assertTrue( propChangedEvents[0] instanceof ResourcePropertyValueChangeNotificationType );
        ResourcePropertyValueChangeNotificationType propChangedEvent = (ResourcePropertyValueChangeNotificationType)propChangedEvents[0];
        assertFalse( propChangedEvent.getNewValue().toString().equals( propChangedEvent.getOldValue().toString() ) );
    }

}
