/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.muws;

import org.apache.commons.id.IdentifierUtils;
import org.apache.commons.id.uuid.UUID;
import org.apache.ws.muws.v1_0.MuwsConstants;
import org.apache.ws.muws.v1_0.capability.IdentityCapability;
import org.apache.ws.muws.v1_0.capability.RelationshipsCapability;
import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.TopicSpace;
import org.apache.ws.notification.topics.TopicSpaceSet;
import org.apache.ws.notification.topics.impl.TopicSpaceImpl;
import org.apache.ws.resource.PropertiesResource;
import org.apache.ws.resource.Resource;
import org.apache.ws.resource.properties.ResourceProperty;
import org.apache.ws.resource.properties.ResourcePropertySet;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart1.ResourceIdDocument;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;

/**
 * A set of utility methods to aid in initializing MUWS manageable resources.
 *
 * @author Sal Campana
 */
public class MuwsUtils
{
   /**
    * Adds the MUWS RelationshipCreated/Deleted topics to the specified topic set.
    *
    * @param topicSet
    */
   public static TopicSpace addRelationshipTopics( TopicSpaceSet topicSet )
   {
      TopicSpace topicSpace = new TopicSpaceImpl( MuwsConstants.NSURI_MUWS_PART2_TOPICS );
      Topic      topic = topicSpace.addTopic( RelationshipsCapability.TOPIC_NAME );
      topic.addTopic( RelationshipsCapability.SUBTOPIC_NAME_RELATIONSHIP_CREATED );
      topic.addTopic( RelationshipsCapability.SUBTOPIC_NAME_RELATIONSHIP_DELETED );
      return topicSet.addTopicSpace( topicSpace );
   }

   /**
    * Utility method for initializing the MUWS Identity ResourceId property.
    *
    * @param resource The resource to add the property to.
    */
   public static void initResourceIdProperty( Resource resource )
   {
      if ( resource instanceof PropertiesResource )
      {
         ResourceProperty resourceProperty =
            ( (PropertiesResource) resource ).getResourcePropertySet(  ).get( IdentityCapability.PROP_NAME_RESOURCE_ID );
         if ( resourceProperty != null )
         {
            if ( resourceProperty.isEmpty(  ) ) // ResourceId prop is immutable, so it only needs to be set once
            {
               URI                idURI    = toURI( resource.getID(  ) );
               ResourceIdDocument propElem = ResourceIdDocument.Factory.newInstance(  );
               propElem.setResourceId( idURI.toString(  ) );
               resourceProperty.add( propElem );
            }
         }
      }
   }

   /**
    * Replaces the Relationship resource property in the specified property set with a
    * special {@link org.apache.ws.muws.v1_0.impl.XmlBeansRelationshipResourceProperty}.
    *
    * @param propSet
    */
   public static void updateRelationshipResourceProperty( ResourcePropertySet                                          propSet,
                                                          org.apache.ws.notification.base.NotificationProducerResource notifResource )
   {
      if ( propSet.get( org.apache.ws.muws.v1_0.capability.RelationshipsCapability.PROP_NAME_RELATIONSHIP ) != null )
      {
         org.apache.ws.muws.v1_0.impl.XmlBeansRelationshipResourceProperty relationshipResourceProperty =
            new org.apache.ws.muws.v1_0.impl.XmlBeansRelationshipResourceProperty( propSet.getMetaData(  )
                                                                                          .getPropertyMetaData( org.apache.ws.muws.v1_0.capability.RelationshipsCapability.PROP_NAME_RELATIONSHIP ),
                                                                                   (org.apache.ws.resource.properties.impl.XmlBeansResourcePropertySet) propSet,
                                                                                   notifResource );
         ( (org.apache.ws.resource.properties.impl.XmlBeansResourcePropertySet) propSet ).set( relationshipResourceProperty );
      }
   }

   /**
    * Returns the base URI to which {@link #toURI} will append a stringified version
    * of a resource identifier. The default implementation returns a URN representation
    * of a v4 UUID.
    *
    * @return the base URI to which {@link #toURI} will append a stringified version
    *         of a resource identifier
    */
   private static String getBaseURI(  )
   {
      try
      {
         return ( (UUID) IdentifierUtils.UUID_VERSION_FOUR_GENERATOR.nextIdentifier(  ) ).toUrn(  );
      }
      catch ( Exception e )
      {
         throw new RuntimeException( "Failed to generate UUID.", e );
      }
   }

   /**
    * Converts the specified resource identifier into a globally unique URI that can
    * be used as the value for the MUWS ResourceId property.
    *
    * @param resourceId a resource identifier
    * @return URI that can be used as the value for the MUWS ResourceId property
    */
   private static URI toURI( Object resourceId )
   {
      try
      {
         return new URI( String.valueOf( resourceId ) );
      }
      catch ( URISyntaxException urise )
      {
         // resource identifier is not already a URI - convert it to one...
      }

      String idString;
      if ( resourceId != null )
      {
         idString = resourceId.toString(  );

         // URL-encode the id in case it contains any unsafe characters
         try
         {
            idString = URLEncoder.encode( idString, "UTF-8" );
         }
         catch ( UnsupportedEncodingException uee )
         {
            // will never happen - all JDKs support UTF-8
            throw new IllegalStateException( uee.getLocalizedMessage(  ) );
         }
      }
      else
      {
         idString = "SINGLETON";
      }

      return URI.create( getBaseURI(  ) + "/" + idString );
   }
}